#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################

from OFS.ObjectManager import ObjectManager

from Products.CMFDocMgt import DocMgtRootPermissions

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import _checkPermission, _getViewFor
#from Products.CMFCore.PortalFolder import PortalFolder

from Products.CMFPlone import transaction_note

from Products.CMFPlone.PloneFolder import PloneFolder
from Products.RevisionManager.Properties import propertyAdapterRegistry
#from Products.RevisionManager.FactoryRegistry import FactoryRegistry
from Products.RevisionManager.util import escape_string, unescape_string

from Globals import HTMLFile, package_home, InitializeClass
from AccessControl import Permissions, ClassSecurityInfo
from Acquisition import aq_parent, aq_inner, aq_base, aq_chain
from ComputedAttribute import ComputedAttribute

from zLOG import LOG, WARNING, BLATHER, INFO

import types
from ZPublisher.Converters import type_converters



import os

# Factory type information -- (for portal_types )
factory_type_information = \
( { 'id'                : 'DocMgtRoot'
  , 'content_icon'      : 'folder_icon.gif'
  , 'meta_type'         : 'DocMgtRoot'
  , 'description'       : 'DocMgtRoot is storing various objects'
                          'which are needed in CMF CVS'
  , 'product'           : 'CMFDocMgt'
  , 'factory'           : 'addDocMgtRoot'
  , 'immediate_view'    : 'docmgtroot_edit_form'
  , 'allowed_content_types'    : ('RevisionManager', 'DocMgtFolder',)
  , 'actions'           :
    ( { 'id'            : 'view'
      , 'name'          : 'View'
      , 'action'        : 'docmgtroot_view'
      , 'permissions'   : (CMFCorePermissions.View, )
      , 'visible':1
      }
    , { 'id'            : 'edit'
      , 'name'          : 'Edit'
      , 'action'        : 'docmgtroot_edit_form'
      , 'permissions'   : (DocMgtRootPermissions.ChangeRoot, )
      , 'visible':1
      }
    , { 'id'            : 'folder_localrole_form'
      , 'name'          : 'Sharing'
      , 'action'        : 'folder_localrole_form'
      , 'permissions'   : (DocMgtRootPermissions.ChangeRoot,)
      , 'category'	: 'object'
      , 'visible':1
      }
    , { 'id'            : 'docmgtroot_revisionmanager'
      , 'name'          : 'RevisionManager'
      , 'action'        : 'docmgtroot_revisionmanager'
      , 'permissions'   : (DocMgtRootPermissions.ChangeRoot,)
      , 'visible':1
      , 'category'	  : 'object'
      }
    , { 'id'            : 'myfiles'
      , 'name'          : 'My Files'
      , 'action'        : 'myfiles'
      , 'permissions'   : (CMFCorePermissions.View,)
      , 'visible':1
      , 'category'	  : 'object'
      }
    )
  }
,
)

def addDocMgtRoot( self, id, title='', REQUEST=None ):
    root = DocMgtRoot( id )
    root.id = id
    root.title = title
    portal_membership = self.portal_membership
    author = portal_membership.getAuthenticatedMember()
    
    root.manage_addProperty('author', author, 'string')
    root.manage_addProperty('email', '', 'string')
    self._setObject( id, root )
    
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( 'manage_main' )


class DocMgtRoot( PloneFolder ):
    
    meta_type='DocMgtRoot'

    security = ClassSecurityInfo()

    security.declareObjectProtected( CMFCorePermissions.View )

    _criteriaTypes = []

    # Contentish interface methods
    # ----------------------------


        
    security.declareProtected( CMFCorePermissions.View, 'icon' )
    def icon( self ):
        """
            For the ZMI.
        """
        return self.getIcon()

    security.declarePrivate( '_verifyActionPermissions' )
    def _verifyActionPermissions( self, action ):
        pp = action.get( 'permissions', () )
        if not pp:
            return 1
        for p in pp:
            if _checkPermission( p, self ):
                return 1
        return 0

    security.declareProtected( DocMgtRootPermissions.ChangeRoot, 'edit' )
    def edit( self, title=None, description=None, author=None, email=None ):

        self.setTitle(title)
        self.setDescription(description)
        self.author = author
        self.email = email
        
        
    #############################################################################
    ##                             RevisionManager                             ##
    #############################################################################

    def findparent(object):
        """ find the direct parent """
    
        for parent in aq_chain(object):
            if parent is object:
                continue
            if parent == object:
                continue
            if isinstance(parent, ObjectManager):
                return parent
        return None    


    security.declarePrivate('deserialize')
    def deserialize(self, serials):
        assert type(serials) is type('')
        properties = []
        lines = serials.split('\n')
        for line in lines:
            if line.startswith('#'):
                continue
            try:
                key, value = line.split('=')
            except ValueError:
                continue
            id, typ = key.split(':')
            id.strip()
            typ.strip()
            value = unescape_string(value)
            try:
                if type_converters.has_key(typ):
                    value = type_converters[typ](value)
                properties.append({
                    'id':    id,
                    'type':  typ,
                    'value': value
                    })
            except:
                continue
        return tuple(properties)


    security.declareProtected(DocMgtRootPermissions.ChangeRoot, "soqrm_mapFSObjects")
    def soqrm_mapFSObjects(self, REQUEST, RESPONSE, items=[]):
        """
        Map file system files to objects.
        """
        failed = []
        for item in items:
            if item.get('add', 0):
                path = item['path']
                rt = self.soqaddFSFileObject(path, item['meta_type'])
                if rt:
                    failed.append(path)
        if failed:
            message = "Object creation failed for: %s" % ' '.join(failed)
            portal_status_message = message
        else:
            portal_status_message = 'Config+Action+Finished+Successfully'
        return REQUEST.RESPONSE.redirect(self.absolute_url().replace('%20',' ') + '?portal_status_message=' + portal_status_message)


    security.declareProtected(DocMgtRootPermissions.ChangeRoot, 'soqaddFSFileObject')
    def soqaddFSFileObject(self, path, meta_type):
        segments = path.split("/")
        rm = self.RevisionManager
        author = self.portal_membership.getAuthenticatedMember()
        # first make sure that the directory part of the path exists
        # if it doesn't, create as many folders as necessary
        folder = aq_inner(self) #aq_parent(aq_inner(self))
        for id in segments[:-1]:
            subfolder = getattr(folder, id, None)
            if subfolder is None:
                folder.invokeFactory(id=id, type_name='DocMgtFolder', title=id)
                subfolder = getattr(folder, id)
            folder = subfolder

        # now add the object
        # first the the appropriate factory
        id = segments[-1]
        abs_filename = rm.getAbsoluteFilePath(path)
        # read the file contents
        data = open(abs_filename, "rb").read()

        self.REQUEST.set('MappingRM', 1)
        if not rm._settings['save_properties']:
            try:
                folder.invokeFactory(id=id, type_name='DocMgtFile')
                obj=getattr(folder, id, None)
                obj.insertProperties(author)
                title = id
                obj.edit(title=title,
                         description='It is ' + title,
                         file=data,
                         author=author,
                         email='')
                #obj.edit(title=title,
                #         description='It is ' + title,
                #         file=data,
                #         author=author,
                #         email='')
            except:
                return 1
            #folder._setObject(id, obj)
            return 0

        basename = os.path.basename(abs_filename)
        dirname = os.path.dirname(abs_filename)
        prop_path = os.path.join(dirname, "." + basename + '.properties')

        properties = ()
        title = id
        props = {}
        # it could be that there is no properties file, so we
        # better check
        if os.path.isfile(prop_path):
            # read the properties file and deserialize the data
            fp = open(prop_path, "rb")
            serial = fp.read()
            fp.close()
            properties = self.deserialize(serial)

            for propdict in properties:
                if propdict['id'] == 'title':
                    title = propdict['value']
                else:
                    props[propdict['id']] = propdict['value']

        # create the object
        try:
            folder.invokeFactory(id=id, type_name='DocMgtFile')
            obj=getattr(folder, id, None)
            
            obj.edit(title=title, 
                     file=data,
                     **props)

        except:
            return 1

        # write properties if available and possible
        if properties:
            adapter = propertyAdapterRegistry.getAdapter(obj)
            if adapter is not None:
                obj_status = rm.getStatus(path)
                obj_status['have_props'] = 1
                if properties != adapter.getProperties(obj):
                    try:
                        adapter.setProperties(obj, properties)
                    except:
                        return 1
        obj.insertProperties(author)
        #folder._setObject(id, obj)
        return 0


    #############################################################################
    ##                             Syndication                                 ##
    #############################################################################

    security.declareProtected( CMFCorePermissions.View, 'queryCatalog' )
    def queryCatalog( self, REQUEST=None, **kw ):
        query = {'path': self.absolute_url('relative').replace('%20',' '),
                'meta_type' :['DocMgtFile'],
                }

        kw.update(query )
        return apply( self.portal_catalog.searchResults, ( REQUEST, ), kw )


    security.declarePublic('synContentValues')
    def synContentValues(self):
        """ play nice with portal_syndication_tool"""
        return self.queryCatalog()

    #############################################################################
    ##                       HTTP (and presumably FTP?) Request                ##
    #############################################################################

    def PUT_factory( self, name, typ, body ):
        """
            Dispatcher for PUT requests to non-existent IDs.  Returns
            an object of the appropriate type (or None, if we don't
            know what to do).
        """
        registry = getToolByName( self, 'content_type_registry' )
        if registry is None:
            return None

        typeObjectName = registry.findTypeName( name, typ, body )
        if typeObjectName is None:
            return None
        if typeObjectName == 'File':
            typeObjectName = 'DocMgtFile'
            
        self.invokeFactory( typeObjectName, name )
        

        # XXX: this is butt-ugly.
        obj = aq_base( self._getOb( name ) )
        self._delObject( name )
        return obj

InitializeClass( DocMgtRoot )
